/*
  IRTele.cpp - Librairie pour grer la telecommande DFRobot.
  Cr par Richard GAUTHIER en Dcembre 2013 pour les lves de la spcialit SIN en STI2D.
  Released into the public domain.
*/

#include "Arduino.h"
#include "IRTele.h"

#define IR_BIT_LENGTH 32    // number of bits sent by IR remote
#define FirstLastBit 15     // divide 32 bits into two 15 bit chunks for integer variables. Ignore center two bits. they are all the same.
#define BIT_1 1500          // Binary 1 threshold (Microseconds)
#define BIT_0 450           // Binary 0 threshold (Microseconds)
#define BIT_START 4000      // Start bit threshold (Microseconds)


IRTele::IRTele(uint8_t IR_PIN)
{
  pinMode(IR_PIN, INPUT);
  _pin = IR_PIN;
}

boolean IRTele::available()
{
     if (pulseIn(_pin, LOW) < BIT_START) return true ;
       else return false ;


}


void IRTele::pulse_to_bits(int pulse[], int bits[])
{

  for(int i = 0; i < IR_BIT_LENGTH; i++)
  {

    if(pulse[i] > BIT_1) //is it a 1?
    {
      bits[i] = 1;
    }
    else if(pulse[i] > BIT_0) //is it a 0?
    {
      bits[i] = 0;
    }
    else //data is invalid...
    {
     // Serial.println("Error");
    }
  }
}


void IRTele::read_pulse(int pulse[])
{
  for (int i = 2; i < IR_BIT_LENGTH; i++)
  {
    pulse[i] = pulseIn(_pin, HIGH);
  }
}

void IRTele::RemoteVerify(int bits[])
{
  int result = 0;
  int seed = 1;

  //Convert bits to integer
  for(int i = 0 ; i < (FirstLastBit) ; i++)
  {
    if(bits[i] == 1)
    {
    result += seed;
    }

    seed *= 2;
  }

}

int IRTele::bits_to_int(int bits[])
{
  int result = 0;
  int seed = 1;

  //Convert bits to integer
  for(int i = (IR_BIT_LENGTH-FirstLastBit) ; i < IR_BIT_LENGTH ; i++)
  {
    if(bits[i] == 1)
    {
    result += seed;
    }
    seed *= 2;
  }
  return result;

}


int IRTele::read()
{
  int pulse[IR_BIT_LENGTH];
  int bits[IR_BIT_LENGTH];

  //do {} //Wait for a start bit
  //while(pulseIn(IR_PIN, HIGH) < BIT_START);

  read_pulse(pulse);
  pulse_to_bits(pulse, bits);
  RemoteVerify(bits);
  return bits_to_int(bits);

}


